package com.baidu.rtc.videoroom;

import android.Manifest;
import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.app.LoaderManager.LoaderCallbacks;
import android.content.CursorLoader;
import android.content.Intent;
import android.content.Loader;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.ContactsContract;
import android.support.annotation.NonNull;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.TextView;

import com.baidu.rtc.videoroom.utils.Utils;

import java.util.ArrayList;
import java.util.List;

/**
 * A login screen that offers login via email/password.
 */
public class LoginActivity extends AppCompatActivity implements LoaderCallbacks<Cursor> {
    private static final String TAG = "LoginActivity";

    private static final int REQUEST_MIC_PERMISSION_ID = 101;

    /**
     * Keep track of the login task to ensure we can cancel it if requested.
     */
    private UserLoginTask mAuthTask = null;

    // UI references.
    private AutoCompleteTextView mUserIDView;
    private EditText mRoomIDView;
    private CheckBox mVideoRoomCheckbox;
    private CheckBox mCallModeCheckbox;
    private View mProgressView;
    private View mLoginFormView;
    private String mUserDisplayname;
    private String mMediaServerIP = "";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON,
                WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.activity_login);
        // Set up the login form.
        mUserIDView = (AutoCompleteTextView) findViewById(R.id.userid);

        int userId = 78657895 + (Build.SERIAL.hashCode() % 100000) + (int) (Math.random() * 10000);

        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);

        mUserDisplayname =
                prefs.getString("user_display_name", getString(R.string.pref_default_display_name));

        mUserIDView.setText(Integer.valueOf(prefs.getInt("UserID", userId)).toString());

        myRequestPermissions();

        mRoomIDView = (EditText) findViewById(R.id.roomid);

        try {
            mRoomIDView.setText(prefs.getString("RoomID", "8888"));
        } catch (Exception e) {
            // update from old version, using int value
            mRoomIDView.setText(Integer.valueOf(prefs.getInt("RoomID", 8888)).toString());
        }

        mRoomIDView.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int id, KeyEvent keyEvent) {
                if (id == EditorInfo.IME_ACTION_DONE || id == EditorInfo.IME_NULL) {
                    attemptLogin();
                    SharedPreferences prefs =
                            PreferenceManager.getDefaultSharedPreferences(LoginActivity.this);

                    prefs.edit().putInt("UserID",
                            Integer.valueOf(mUserIDView.getText().toString()).intValue()).commit();

                    if (mRoomIDView.getText().toString().isEmpty()) {
                        return true;
                    }
                    prefs.edit().putString("RoomID", mRoomIDView.getText().toString()).commit();
                    return true;
                }
                return false;
            }
        });

        Button mSignInButton = (Button) findViewById(R.id.sign_in_button);
        mSignInButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!myRequestPermissions()) {
                    Utils.makeToast(LoginActivity.this, "no permission!");
                    return;
                }
                attemptLogin();
                SharedPreferences prefs =
                        PreferenceManager.getDefaultSharedPreferences(LoginActivity.this);

                prefs.edit().putInt("UserID",
                        Integer.valueOf(mUserIDView.getText().toString()).intValue()).commit();

                if (mRoomIDView.getText().toString().isEmpty()) {
                    return;
                }
                prefs.edit().putString("RoomID", mRoomIDView.getText().toString()).commit();
            }
        });

        Button mSettingsButton = (Button) findViewById(R.id.settings_button);
        mSettingsButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(LoginActivity.this, SettingsActivity.class);
                startActivity(intent);
            }
        });

        mVideoRoomCheckbox = (CheckBox) findViewById(R.id.checkbox_videoroom);

        mVideoRoomCheckbox.setChecked(prefs.getBoolean("video_room_checked", false));

        mVideoRoomCheckbox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    if (mCallModeCheckbox != null) {
                        mCallModeCheckbox.setChecked(false);
                    }
                }
                SharedPreferences prefs =
                        PreferenceManager.getDefaultSharedPreferences(LoginActivity.this);
                prefs.edit().putBoolean("video_room_checked", isChecked).commit();
            }
        });

        mCallModeCheckbox = (CheckBox) findViewById(R.id.checkbox_callMode);
        mCallModeCheckbox.setChecked(prefs.getBoolean("call_mode_checked", false));
        mCallModeCheckbox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                if (isChecked) {
                    mVideoRoomCheckbox.setChecked(false);
                }
                SharedPreferences prefs =
                        PreferenceManager.getDefaultSharedPreferences(LoginActivity.this);
                prefs.edit().putBoolean("call_mode_checked", isChecked).commit();
            }
        });

        mLoginFormView = findViewById(R.id.login_form);
        mProgressView = findViewById(R.id.login_progress);
    }

    private boolean myRequestPermissions() {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            return true;
        }

        int audio = checkSelfPermission(Manifest.permission.RECORD_AUDIO);
        int sdcard = checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE);
        List<String> listPermissionsNeeded = new ArrayList<>();

        if (audio != PackageManager.PERMISSION_GRANTED) {
            listPermissionsNeeded.add(Manifest.permission.RECORD_AUDIO);
        }

        if( sdcard != PackageManager.PERMISSION_GRANTED) {
            listPermissionsNeeded.add(android.Manifest.permission.WRITE_EXTERNAL_STORAGE);
        }

        if (!listPermissionsNeeded.isEmpty()) {
            requestPermissions(listPermissionsNeeded.toArray
                    (new String[listPermissionsNeeded.size()]), REQUEST_MIC_PERMISSION_ID);
            return false;
        }
        return true;
    }

    /**
     * Callback received when a permissions request has been completed.
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        if (requestCode == REQUEST_MIC_PERMISSION_ID) {
            if (grantResults.length == 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                myRequestPermissions();
            }
        }
    }


    /**
     * Attempts to sign in or register the account specified by the login form.
     * If there are form errors (invalid email, missing fields, etc.), the
     * errors are presented and no actual login attempt is made.
     */
    private void attemptLogin() {
        if (mAuthTask != null) {
            return;
        }

        // Reset errors.
        mUserIDView.setError(null);
        mRoomIDView.setError(null);

        // Store values at the time of the login attempt.
        String userid = mUserIDView.getText().toString();
        String roomid = mRoomIDView.getText().toString();

        boolean cancel = false;
        View focusView = null;

        // Check for a valid password, if the user entered one.
        if (TextUtils.isEmpty(roomid)) {
            mRoomIDView.setError(getString(R.string.error_invalid_roomid));
            focusView = mRoomIDView;
            cancel = true;
        }

        // Check for a valid userid address.
        if (TextUtils.isEmpty(userid)) {
            mUserIDView.setError(getString(R.string.error_field_required));
            focusView = mUserIDView;
            cancel = true;
        }

        if (cancel) {
            // There was an error; don't attempt login and focus the first
            // form field with an error.
            if (focusView != null) {
                focusView.requestFocus();
            }
        } else {
            // Show a progress spinner, and kick off a background task to
            // perform the user login attempt.
            showProgress(true);
            mAuthTask = new UserLoginTask(userid, roomid);
            mAuthTask.execute((Void) null);
        }
    }

    /**
     * Shows the progress UI and hides the login form.
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
    private void showProgress(final boolean show) {
        // On Honeycomb MR2 we have the ViewPropertyAnimator APIs, which allow
        // for very easy animations. If available, use these APIs to fade-in
        // the progress spinner.
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
            int shortAnimTime = getResources().getInteger(android.R.integer.config_shortAnimTime);

            mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
            mLoginFormView.animate().setDuration(shortAnimTime).alpha(
                    show ? 0 : 1).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
                }
            });

            mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            mProgressView.animate().setDuration(shortAnimTime).alpha(
                    show ? 1 : 0).setListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
                }
            });
        } else {
            // The ViewPropertyAnimator APIs are not available, so simply show
            // and hide the relevant UI components.
            mProgressView.setVisibility(show ? View.VISIBLE : View.GONE);
            mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
        }
    }

    @Override
    public Loader<Cursor> onCreateLoader(int i, Bundle bundle) {
        return new CursorLoader(this,
                // Retrieve data rows for the device user's 'profile' contact.
                Uri.withAppendedPath(ContactsContract.Profile.CONTENT_URI,
                        ContactsContract.Contacts.Data.CONTENT_DIRECTORY), ProfileQuery.PROJECTION,

                // Select only userid addresses.
                ContactsContract.Contacts.Data.MIMETYPE +
                        " = ?", new String[]{ContactsContract.CommonDataKinds.Email
                .CONTENT_ITEM_TYPE},

                // Show primary email addresses first. Note that there won't be
                // a primary email address if the user hasn't specified one.
                ContactsContract.Contacts.Data.IS_PRIMARY + " DESC");
    }

    @Override
    public void onLoadFinished(Loader<Cursor> cursorLoader, Cursor cursor) {
        List<String> emails = new ArrayList<>();
        cursor.moveToFirst();
        while (!cursor.isAfterLast()) {
            emails.add(cursor.getString(ProfileQuery.ADDRESS));
            cursor.moveToNext();
        }

        addEmailsToAutoComplete(emails);
    }

    @Override
    public void onLoaderReset(Loader<Cursor> cursorLoader) {

    }

    private void addEmailsToAutoComplete(List<String> emailAddressCollection) {
        // Create adapter to tell the AutoCompleteTextView what to show in its dropdown list.
        ArrayAdapter<String> adapter =
                new ArrayAdapter<>(LoginActivity.this,
                        android.R.layout.simple_dropdown_item_1line, emailAddressCollection);

        mUserIDView.setAdapter(adapter);
    }

    private interface ProfileQuery {
        String[] PROJECTION = {
                ContactsContract.CommonDataKinds.Email.ADDRESS,
                ContactsContract.CommonDataKinds.Email.IS_PRIMARY,
        };

        int ADDRESS = 0;
        int IS_PRIMARY = 1;
    }

    /**
     * Represents an asynchronous login/registration task used to authenticate
     * the user.
     */
    public class UserLoginTask extends AsyncTask<Void, Void, Boolean> {

        private final String muserid;
        private String mRoomName = "";

        UserLoginTask(String userid, String roomid) {
            muserid = userid;
            mRoomName = roomid;
        }

        @Override
        protected Boolean doInBackground(Void... params) {
            // TODO: attempt authentication against a network service.

            return true;
        }

        @Override
        protected void onPostExecute(final Boolean success) {
            mAuthTask = null;
            showProgress(false);

            if (success) {
                Intent intent = new Intent(LoginActivity.this,
                        mVideoRoomCheckbox.isChecked() ? RoomActivity.class
                                : RtcCommonActivity.class);
                intent.putExtra("callMode", mCallModeCheckbox.isChecked() ? true : false);
                intent.putExtra("userid", muserid);
                intent.putExtra("roomname", mRoomName);
                SharedPreferences prefs =
                        PreferenceManager.getDefaultSharedPreferences(LoginActivity.this);
                String MediaServerUrl = prefs.getString("key_media_server_url",
                        getString(R.string.pref_default_media_server_url));

//                boolean audio_only = prefs.getBoolean("key_audio_only", false);

                intent.putExtra("audio_only", true);

                boolean disableBuildInNS = prefs.getBoolean("key_disable_builtin_ns", false);
                intent.putExtra("disable_builtin_ns", disableBuildInNS);

//                String video_resolution = prefs.getString("video_resolution", "640x480-1000kbps");
//                intent.putExtra("video_resolution", video_resolution);

                mUserDisplayname = prefs.getString("user_display_name",
                        getString(R.string.pref_default_display_name));

                intent.putExtra("username", mUserDisplayname);

                intent.putExtra("mediaserver", MediaServerUrl);

                mMediaServerIP = prefs.getString("key_media_server_ip", "");
                intent.putExtra("mediaserverip", mMediaServerIP);

                String appid = prefs.getString("key_appid", "");
                intent.putExtra("appid", appid);

//                boolean datachannel_test = prefs.getBoolean("key_datachannel_test", false);
//                intent.putExtra("datachannel_test", datachannel_test);

                boolean autoSubscribeStream = prefs.getBoolean("key_auto_subscribe_stream", true);
                intent.putExtra("key_auto_subscribe_stream", autoSubscribeStream);
                Log.i(TAG, "key_auto_subscribe_stream = " + autoSubscribeStream);

                boolean autoPublishStream = prefs.getBoolean("key_auto_publish_stream", true);
                intent.putExtra("key_auto_publish_stream", autoPublishStream);
                Log.i(TAG, "key_auto_publish_stream = " + autoPublishStream);

                boolean stream_monitor_data_report = prefs.getBoolean("key_stream_data_report_test", false);
                intent.putExtra("stream_monitor_data_report", stream_monitor_data_report);

                String rtmp_url = prefs.getString("key_rtmp_url", "");
                intent.putExtra("rtmp_url", rtmp_url);

                boolean rtmp_mix = prefs.getBoolean("key_rtmp_mix", false);
                intent.putExtra("rtmp_mix", rtmp_mix);

                boolean recording = prefs.getBoolean("key_recording", false);
                intent.putExtra("recording", recording);

                startActivity(intent);
            } else {
                mRoomIDView.setError(getString(R.string.error_incorrect_roomid));
                mRoomIDView.requestFocus();
            }
        }

        @Override
        protected void onCancelled() {
            mAuthTask = null;
            showProgress(false);
        }
    }
}

