package com.baidu.rtc.videoroom;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.GridLayoutManager;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.baidu.rtc.BaiduRtcRoom;
import com.baidu.rtc.BaiduRtcRoomDelegate;
import com.baidu.rtc.videoroom.model.RoomData;
import com.baidu.rtc.videoroom.model.UserModel;

public class RoomActivity extends Activity {
    private static final String TAG = "RoomActivity";
    RecyclerView remoteView;
    TextView mHeaderView;
    private Handler volumeHandler = new Handler(Looper.getMainLooper());
    private UserModel mUserModel = new UserModel();

    private BRtcWrapper rtcWrapper;
    private RoomData mData;

    BRtcWrapper.RtcListener mListener = new BRtcWrapper.RtcListener() {
        @Override
        public void startLogin() {
            if (RoomActivity.this.isDestroyed()) {
                return;
            }
            syncUIState();
            updateRoomState(UserModel.RoomState.LOGIN);
        }

        @Override
        public void onLoginRtcRoom(int code, long roomId) {
            if (RoomActivity.this.isDestroyed()) {
                return;
            }

            if (code == 0) {
                updateRoomState(UserModel.RoomState.OK);
                if (isPulling) {
                    return;
                }
                isPulling = true;
                updateVolume();
            } else {
                updateRoomState(UserModel.RoomState.FAIL);
            }
            StringBuffer sb = new StringBuffer();
            sb.append("uid:" + mData.mUserId);
            sb.append(" | roomId:" + mData.mRoomName);
            updateHeaderView(sb.toString());
        }

        @Override
        public void onConnectionLost(int code) {
            updateRoomState(UserModel.RoomState.IDLE);
        }

        @Override
        public void onPublisherMediaStateUpdate(int state) {
            if (state == BaiduRtcRoomDelegate.RTC_STATE_ICE_CONNECTED) {
                updateStreamState(UserModel.StreamState.ICE_CONNECTED);
            } else if (state == BaiduRtcRoomDelegate.RTC_STATE_STREAM_UP) {
                updateStreamState(UserModel.StreamState.WEBRTC_UP);
            } else if (state == BaiduRtcRoomDelegate.RTC_STATE_ICE_DISCONNECTED) {
                updateStreamState(UserModel.StreamState.ICE_DIS_CONNECTED);
            } else if (state == BaiduRtcRoomDelegate.RTC_STATE_SENDING_MEDIA_OK) {
                updateStreamState(UserModel.StreamState.SEND_MEDIA_OK);
            } else if (state == BaiduRtcRoomDelegate.RTC_STATE_SENDING_MEDIA_FAILED ||
                    state == BaiduRtcRoomDelegate.RTC_STATE_PUBLISHER_STREAM_DOWN) {
                updateStreamState(UserModel.StreamState.SEND_MEDIA_FAIL);
            }
        }


        @Override
        public void onRemoteStreamComing(long uid, String name) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mUserModel.add(uid);
                    mUserModel.update();
                }
            });
        }

        @Override
        public void onRemoteStreamLeaving(long uid) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mUserModel.remove(uid);
                    mUserModel.update();
                }
            });
        }

        @Override
        public void requestExist() {
            RoomActivity.this.finish();
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        rtcWrapper = new BRtcWrapper(mListener);
        mData = RoomData.createFromIntent(getIntent(), this);
        rtcWrapper.onCreate(mData, this);

        setContentView(R.layout.activity_videoroom);

        remoteView = (RecyclerView) findViewById(R.id.recycle_view);
        remoteView.setLayoutManager(new LinearLayoutManager(this));
        remoteView.setLayoutManager(new GridLayoutManager(this, 2));
        remoteView.addItemDecoration(new DividerItemDecoration(this, DividerItemDecoration.VERTICAL));
        mHeaderView = findViewById(R.id.nav_header);
        UserModel.UserInfo userInfo = new UserModel.UserInfo();
        userInfo.userId = Long.valueOf(mData.mUserId);
        userInfo.role = UserModel.UserRole.LOCAL;
        mUserModel.add(userInfo);
        mUserModel.initView(this, remoteView);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (rtcWrapper != null) {
            rtcWrapper.onDestroy();
            rtcWrapper = null;
        }
        if (volumeHandler != null) {
            volumeHandler.removeCallbacksAndMessages(null);
        }
        Log.i(TAG, "onDestroy");
    }

    private void updateRoomState(UserModel.RoomState state) {
        try {
            long uid = Integer.valueOf(mData.mUserId);
            UserModel.UserInfo userInfo = mUserModel.get(uid);
            if (userInfo != null) {
                userInfo.roomState = state;
                mUserModel.update();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateStreamState(UserModel.StreamState state) {
        try {
            long uid = Integer.valueOf(mData.mUserId);
            UserModel.UserInfo userInfo = mUserModel.get(uid);
            if (userInfo != null) {
                userInfo.streamState = state;
                mUserModel.update();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateHeaderView(String txt) {
        if (mHeaderView == null) {
            return;
        }
        if (Looper.getMainLooper() != Looper.myLooper()) {
            volumeHandler.post(new Runnable() {
                @Override
                public void run() {
                    mHeaderView.setText(txt);
                }
            });
        } else {
            mHeaderView.setText(txt);
        }
    }

    boolean isPulling = false;
    private void updateVolume() {
        volumeHandler.postDelayed(new Runnable() {
            @Override
            public void run() {

                if (!rtcWrapper.isConnected()) {
                    updateVolume();
                    return;
                }
                BaiduRtcRoom.RtcRoomAudioLevel localAudio = rtcWrapper.getLocalAudioLevel();
                BaiduRtcRoom.RtcRoomAudioLevel[] remoteAudio = rtcWrapper.getRemoteAudioLevels();

                UserModel.UserInfo info = mUserModel.get(Long.valueOf(mData.mUserId));
                if (info != null) {
                    info.audioVolume = localAudio.volumeLevel;
                }
                mUserModel.updateVolume(remoteAudio);
                mUserModel.update();
                updateVolume();
            }
        }, 200);
    }

    boolean isMutePhone = false;
    public void onLocalAudioMuteClicked(View view) {
        ImageView iv = (ImageView) view;
        if (iv.isSelected()) {
            iv.setSelected(false);
            iv.setImageResource(R.drawable.btn_mute);
        } else {
            iv.setSelected(true);
            iv.setImageResource(R.drawable.btn_mute_selected);
        }
        isMutePhone = iv.isSelected();
        rtcWrapper.muteMicphone(iv.isSelected());
        Log.d(TAG, "onLocalAudioMuteClicked isMutePhone:" + isMutePhone);
    }

    boolean isMuteSpeaker = false;
    public void onSpeakerClicked(View view) {
        ImageView iv = (ImageView) view;
        if (iv.isSelected()) {
            iv.setSelected(false);
            iv.setImageResource(R.drawable.btn_speaker);
        } else {
            iv.setSelected(true);
            iv.setImageResource(R.drawable.btn_speaker_selected);
        }
        isMuteSpeaker = !isMuteSpeaker;
        rtcWrapper.switchLoundSpeaker();
        Log.d(TAG, "onLocalAudioMuteClicked isMuteSpeaker:" + isMuteSpeaker);
    }

    public void onEndCallClicked(View view) {
        rtcWrapper.onDestroy();
        finish();
    }


    void syncUIState() {
        if (isMuteSpeaker) {
            rtcWrapper.switchLoundSpeaker();
        }
        rtcWrapper.muteMicphone(isMutePhone);
    }
}