package com.baidu.rtc.videoroom;

import android.app.Activity;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.ImageView;

import com.baidu.rtc.videoroom.model.RoomData;

public class RtcCommonActivity extends Activity  {
    private static final String TAG = "RtcCommonActivity";
    private BRtcWrapper rtcWrapper;
    private RoomData mData;

    BRtcWrapper.RtcListener mListener = new BRtcWrapper.RtcListener() {
        @Override
        public void onConnectionLost(int code) {

        }
        @Override
        public void onPublisherMediaStateUpdate(int state) {

        }
        @Override
        public void startLogin() {
            if (RtcCommonActivity.this.isDestroyed()) {
                return;
            }
            syncUIState();
        }

        @Override
        public void onLoginRtcRoom(int code, long roomId) {

        }
        @Override
        public void onRemoteStreamComing(long uid, String name) {

        }

        @Override
        public void onRemoteStreamLeaving(long uid) {

        }

        @Override
        public void requestExist() {
            RtcCommonActivity.this.finish();
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON,
                WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        setContentView(R.layout.activity_videocall);
        rtcWrapper = new BRtcWrapper(mListener);
        mData = RoomData.createFromIntent(getIntent(), this);
        rtcWrapper.onCreate(mData, this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        hideBottomUIMenu();
    }

    @Override
    protected void onDestroy() {
        if (rtcWrapper != null) {
            rtcWrapper.onDestroy();
            rtcWrapper = null;
        }
        super.onDestroy();
        Log.i(TAG, "onDestroy");
    }


    boolean isMutePhone = false;
    public void onLocalAudioMuteClicked(View view) {
        ImageView iv = (ImageView) view;
        if (iv.isSelected()) {
            iv.setSelected(false);
            iv.setImageResource(R.drawable.btn_mute);
        } else {
            iv.setSelected(true);
            iv.setImageResource(R.drawable.btn_mute_selected);
        }
        isMutePhone = iv.isSelected();
        rtcWrapper.muteMicphone(iv.isSelected());
        Log.d(TAG, "onLocalAudioMuteClicked isMutePhone:" + isMutePhone);
    }

    boolean isMuteSpeaker = false;
    public void onSpeakerClicked(View view) {
        ImageView iv = (ImageView) view;
        if (iv.isSelected()) {
            iv.setSelected(false);
            iv.setImageResource(R.drawable.btn_speaker);
        } else {
            iv.setSelected(true);
            iv.setImageResource(R.drawable.btn_speaker_selected);
        }
        isMuteSpeaker = !isMuteSpeaker;
        rtcWrapper.switchLoundSpeaker();
        Log.d(TAG, "onLocalAudioMuteClicked isMuteSpeaker:" + isMuteSpeaker);
    }

    public void onEndCallClicked(View view) {
        rtcWrapper.logout();
        rtcWrapper.destroy();
        finish();
    }


    void syncUIState() {
        if (isMuteSpeaker) {
            rtcWrapper.switchLoundSpeaker();
        }
        rtcWrapper.muteMicphone(isMutePhone);
    }

    protected void hideBottomUIMenu() {
        // full screen no bottomUI.
        if (Build.VERSION.SDK_INT > 11 && Build.VERSION.SDK_INT < 19) { // lower api
            View v = this.getWindow().getDecorView();
            v.setSystemUiVisibility(View.GONE);
        } else if (Build.VERSION.SDK_INT >= 19) {
            // for new api versions.
            View decorView = getWindow().getDecorView();
            int uiOptions = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY | View.SYSTEM_UI_FLAG_FULLSCREEN;
            decorView.setSystemUiVisibility(uiOptions);

        }
    }


}
